package java.awt;

import javax.accessibility.AccessibleState;
import javax.accessibility.AccessibleStateSet;
import javax.accessibility.AccessibleRole;
import javax.accessibility.AccessibleContext;
import java.io.IOException;
import java.security.AccessControlException;
import java.io.ObjectInputStream;
import sun.security.util.SecurityConstants;
import java.util.List;
import sun.awt.util.IdentityLinkedList;
import java.util.Iterator;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.awt.event.InvocationEvent;
import sun.awt.AppContext;
import java.awt.event.ComponentEvent;
import java.lang.ref.WeakReference;
import java.util.concurrent.atomic.AtomicLong;
import java.awt.peer.DialogPeer;
import sun.awt.SunToolkit;
import sun.awt.util.IdentityArrayList;

public class Dialog extends Window
{
    boolean resizable;
    boolean undecorated;
    private transient boolean initialized;
    public static final Dialog.ModalityType DEFAULT_MODALITY_TYPE;
    boolean modal;
    Dialog.ModalityType modalityType;
    static transient IdentityArrayList<Dialog> modalDialogs;
    transient IdentityArrayList<Window> blockedWindows;
    String title;
    private transient ModalEventFilter modalFilter;
    private transient volatile SecondaryLoop secondaryLoop;
    transient volatile boolean isInHide;
    transient volatile boolean isInDispose;
    private static final String base = "dialog";
    private static int nameCounter;
    private static final long serialVersionUID = 5920926903803293709L;
    
    public Dialog(Frame frame) {
        this(frame, "", false);
    }
    
    public Dialog(Frame frame, boolean b) {
        this(frame, "", b);
    }
    
    public Dialog(Frame frame, String s) {
        this(frame, s, false);
    }
    
    public Dialog(Frame frame, String s, boolean b) {
        this(frame, s, b ? Dialog.DEFAULT_MODALITY_TYPE : Dialog.ModalityType.MODELESS);
    }
    
    public Dialog(Frame frame, String s, boolean b, GraphicsConfiguration graphicsConfiguration) {
        this(frame, s, b ? Dialog.DEFAULT_MODALITY_TYPE : Dialog.ModalityType.MODELESS, graphicsConfiguration);
    }
    
    public Dialog(Dialog dialog) {
        this(dialog, "", false);
    }
    
    public Dialog(Dialog dialog, String s) {
        this(dialog, s, false);
    }
    
    public Dialog(Dialog dialog, String s, boolean b) {
        this(dialog, s, b ? Dialog.DEFAULT_MODALITY_TYPE : Dialog.ModalityType.MODELESS);
    }
    
    public Dialog(Dialog dialog, String s, boolean b, GraphicsConfiguration graphicsConfiguration) {
        this(dialog, s, b ? Dialog.DEFAULT_MODALITY_TYPE : Dialog.ModalityType.MODELESS, graphicsConfiguration);
    }
    
    public Dialog(Window window) {
        this(window, "", Dialog.ModalityType.MODELESS);
    }
    
    public Dialog(Window window, String s) {
        this(window, s, Dialog.ModalityType.MODELESS);
    }
    
    public Dialog(Window window, Dialog.ModalityType modalityType) {
        this(window, "", modalityType);
    }
    
    public Dialog(Window window, String title, Dialog.ModalityType modalityType) {
        super(window);
        this.resizable = true;
        this.undecorated = false;
        this.initialized = false;
        this.blockedWindows = new IdentityArrayList();
        this.isInHide = false;
        this.isInDispose = false;
        if (window != null && !(window instanceof Frame) && !(window instanceof Dialog)) {
            throw new IllegalArgumentException("Wrong parent window");
        }
        this.title = title;
        this.setModalityType(modalityType);
        SunToolkit.checkAndSetPolicy(this);
        this.initialized = true;
        return;
    }
    
    public Dialog(Window window, String title, Dialog.ModalityType modalityType, GraphicsConfiguration graphicsConfiguration) {
        super(window, graphicsConfiguration);
        this.resizable = true;
        this.undecorated = false;
        this.initialized = false;
        this.blockedWindows = new IdentityArrayList();
        this.isInHide = false;
        this.isInDispose = false;
        if (window != null && !(window instanceof Frame) && !(window instanceof Dialog)) {
            throw new IllegalArgumentException("wrong owner window");
        }
        this.title = title;
        this.setModalityType(modalityType);
        SunToolkit.checkAndSetPolicy(this);
        this.initialized = true;
        return;
    }
    
    @Override
    String constructComponentName() {
        synchronized (Dialog.class) {
            return "dialog" + Dialog.nameCounter++;
        }
    }
    
    @Override
    public void addNotify() {
        synchronized (this.getTreeLock()) {
            if (this.parent != null && this.parent.getPeer() == null) {
                this.parent.addNotify();
            }
            if (this.peer == null) {
                this.peer = this.getToolkit().createDialog(this);
            }
            super.addNotify();
        }
    }
    
    public boolean isModal() {
        return this.isModal_NoClientCode();
    }
    
    final boolean isModal_NoClientCode() {
        return this.modalityType != Dialog.ModalityType.MODELESS;
    }
    
    public void setModal(boolean modal) {
        this.modal = modal;
        this.setModalityType(modal ? Dialog.DEFAULT_MODALITY_TYPE : Dialog.ModalityType.MODELESS);
    }
    
    public Dialog.ModalityType getModalityType() {
        return this.modalityType;
    }
    
    public void setModalityType(Dialog.ModalityType modalityType) {
        if (modalityType == null) {
            modalityType = Dialog.ModalityType.MODELESS;
        }
        if (!Toolkit.getDefaultToolkit().isModalityTypeSupported(modalityType)) {
            modalityType = Dialog.ModalityType.MODELESS;
        }
        if (this.modalityType == modalityType) {
            return;
        }
        this.checkModalityPermission(modalityType);
        this.modalityType = modalityType;
        this.modal = (this.modalityType != Dialog.ModalityType.MODELESS);
        return;
    }
    
    public String getTitle() {
        return this.title;
    }
    
    public void setTitle(String s) {
        String title = this.title;
        synchronized (this) {
            this.title = s;
            DialogPeer dialogPeer = (DialogPeer)this.peer;
            if (dialogPeer != null) {
                dialogPeer.setTitle(s);
            }
        }
        this.firePropertyChange("title", title, s);
    }
    
    private boolean conditionalShow(Component component, AtomicLong atomicLong) {
        this.closeSplashScreen();
        int n;
        synchronized (this.getTreeLock()) {
            if (this.peer == null) {
                this.addNotify();
            }
            this.validateUnconditionally();
            if (this.visible) {
                this.toFront();
                n = false;
            }
            else {
                n = (this.visible = true);
                if (!this.isModal()) {
                    checkShouldBeBlocked(this);
                }
                else {
                    Dialog.modalDialogs.add(this);
                    this.modalShow();
                }
                if (component != null && atomicLong != null && this.isFocusable() && this.isEnabled() && !this.isModalBlocked()) {
                    atomicLong.set(Toolkit.getEventQueue().getMostRecentKeyEventTime());
                    KeyboardFocusManager.getCurrentKeyboardFocusManager().enqueueKeyEvents(atomicLong.get(), component);
                }
                this.mixOnShowing();
                this.peer.setVisible(true);
                if (this.isModalBlocked()) {
                    this.modalBlocker.toFront();
                }
                this.setLocationByPlatform(false);
                for (int i = 0; i < this.ownedWindowList.size(); ++i) {
                    Window window = (Window)((WeakReference)this.ownedWindowList.elementAt(i)).get();
                    if (window != null && window.showWithParent) {
                        window.show();
                        window.showWithParent = false;
                    }
                }
                Window.updateChildFocusableWindowState(this);
                this.createHierarchyEvents(1400, this, this.parent, 4L, Toolkit.enabledOnToolkit(32768L));
                if (this.componentListener != null || (this.eventMask & 0x1L) != 0x0L || Toolkit.enabledOnToolkit(1L)) {
                    Toolkit.getEventQueue().postEvent(new ComponentEvent(this, 102));
                }
            }
        }
        if (n && (this.state & 0x1) == 0x0) {
            this.postWindowEvent(200);
            this.state |= 0x1;
        }
        return n;
    }
    
    @Override
    public void setVisible(boolean visible) {
        super.setVisible(visible);
    }
    
    @Deprecated
    @Override
    public void show() {
        if (!this.initialized) {
            throw new IllegalStateException("The dialog component has not been initialized properly");
        }
        this.beforeFirstShow = false;
        if (!this.isModal()) {
            this.conditionalShow(null, null);
        }
        else {
            AppContext appContext = AppContext.getAppContext();
            AtomicLong atomicLong = new AtomicLong();
            Component mostRecentFocusOwner = null;
            try {
                mostRecentFocusOwner = this.getMostRecentFocusOwner();
                if (this.conditionalShow(mostRecentFocusOwner, atomicLong)) {
                    this.modalFilter = ModalEventFilter.createFilterForDialog(this);
                    Conditional conditional = new Conditional() {
                        @Override
                        public boolean evaluate() {
                            return Dialog.this.windowClosingException == null;
                        }
                    };
                    if (this.modalityType == Dialog.ModalityType.TOOLKIT_MODAL) {
                        for (AppContext appContext2 : AppContext.getAppContexts()) {
                            if (appContext2 == appContext) {
                                continue;
                            }
                            EventQueue eventQueue = (EventQueue)appContext2.get(AppContext.EVENT_QUEUE_KEY);
                            eventQueue.postEvent(new InvocationEvent(this, new Runnable() {
                                @Override
                                public void run() {
                                }
                            }));
                            eventQueue.getDispatchThread().addEventFilter(this.modalFilter);
                        }
                    }
                    this.modalityPushed();
                    try {
                        this.secondaryLoop = ((EventQueue)AccessController.doPrivileged(new PrivilegedAction<EventQueue>() {
                            @Override
                            public EventQueue run() {
                                return Toolkit.getDefaultToolkit().getSystemEventQueue();
                            }
                        })).createSecondaryLoop(conditional, this.modalFilter, 0L);
                        if (!this.secondaryLoop.enter()) {
                            this.secondaryLoop = null;
                        }
                    }
                    finally {
                        this.modalityPopped();
                    }
                    if (this.modalityType == Dialog.ModalityType.TOOLKIT_MODAL) {
                        for (AppContext appContext3 : AppContext.getAppContexts()) {
                            if (appContext3 == appContext) {
                                continue;
                            }
                            ((EventQueue)appContext3.get(AppContext.EVENT_QUEUE_KEY)).getDispatchThread().removeEventFilter(this.modalFilter);
                        }
                    }
                    if (this.windowClosingException != null) {
                        this.windowClosingException.fillInStackTrace();
                        throw this.windowClosingException;
                    }
                }
            }
            finally {
                if (mostRecentFocusOwner != null) {
                    KeyboardFocusManager.getCurrentKeyboardFocusManager().dequeueKeyEvents(atomicLong.get(), mostRecentFocusOwner);
                }
            }
        }
        return;
    }
    
    final void modalityPushed() {
        Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
        if ((defaultToolkit instanceof SunToolkit)) {
            ((SunToolkit)defaultToolkit).notifyModalityPushed(this);
        }
    }
    
    final void modalityPopped() {
        Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
        if ((defaultToolkit instanceof SunToolkit)) {
            ((SunToolkit)defaultToolkit).notifyModalityPopped(this);
        }
    }
    
    void interruptBlocking() {
        if (this.isModal()) {
            this.disposeImpl();
        }
        else if (this.windowClosingException != null) {
            this.windowClosingException.fillInStackTrace();
            this.windowClosingException.printStackTrace();
            this.windowClosingException = null;
        }
    }
    
    private void hideAndDisposePreHandler() {
        this.isInHide = true;
        synchronized (this.getTreeLock()) {
            if (this.secondaryLoop != null) {
                this.modalHide();
                if (this.modalFilter != null) {
                    this.modalFilter.disable();
                }
                Dialog.modalDialogs.remove(this);
            }
        }
    }
    
    private void hideAndDisposeHandler() {
        if (this.secondaryLoop != null) {
            this.secondaryLoop.exit();
            this.secondaryLoop = null;
        }
        this.isInHide = false;
    }
    
    @Deprecated
    @Override
    public void hide() {
        this.hideAndDisposePreHandler();
        super.hide();
        if (!this.isInDispose) {
            this.hideAndDisposeHandler();
        }
    }
    
    @Override
    void doDispose() {
        this.isInDispose = true;
        super.doDispose();
        this.hideAndDisposeHandler();
        this.isInDispose = false;
    }
    
    @Override
    public void toBack() {
        super.toBack();
        if (this.visible) {
            synchronized (this.getTreeLock()) {
                Iterator<Object> iterator = this.blockedWindows.iterator();
                while (iterator.hasNext()) {
                    ((Window)iterator.next()).toBack_NoClientCode();
                }
            }
        }
    }
    
    public boolean isResizable() {
        return this.resizable;
    }
    
    public void setResizable(boolean b) {
        boolean n = false;
        synchronized (this) {
            this.resizable = b;
            DialogPeer dialogPeer = (DialogPeer)this.peer;
            if (dialogPeer != null) {
                dialogPeer.setResizable(b);
                n = true;
            }
        }
        if (!(n == 0)) {
            this.invalidateIfValid();
        }
    }
    
    public void setUndecorated(boolean undecorated) {
        synchronized (this.getTreeLock()) {
            if (this.isDisplayable()) {
                throw new IllegalComponentStateException("The dialog is displayable.");
            }
            if (!undecorated) {
                if (this.getOpacity() < 1.0f) {
                    throw new IllegalComponentStateException("The dialog is not opaque");
                }
                if (this.getShape() != null) {
                    throw new IllegalComponentStateException("The dialog does not have a default shape");
                }
                Color background = this.getBackground();
                if (background != null && background.getAlpha() < 255) {
                    throw new IllegalComponentStateException("The dialog background color is not opaque");
                }
            }
            this.undecorated = undecorated;
        }
    }
    
    public boolean isUndecorated() {
        return this.undecorated;
    }
    
    @Override
    public void setOpacity(float opacity) {
        synchronized (this.getTreeLock()) {
            if (opacity < 1.0f && !this.isUndecorated()) {
                throw new IllegalComponentStateException("The dialog is decorated");
            }
            super.setOpacity(opacity);
        }
    }
    
    @Override
    public void setShape(Shape shape) {
        synchronized (this.getTreeLock()) {
            if (shape != null && !this.isUndecorated()) {
                throw new IllegalComponentStateException("The dialog is decorated");
            }
            super.setShape(shape);
        }
    }
    
    @Override
    public void setBackground(Color background) {
        synchronized (this.getTreeLock()) {
            if (background != null && background.getAlpha() < 255 && !this.isUndecorated()) {
                throw new IllegalComponentStateException("The dialog is decorated");
            }
            super.setBackground(background);
        }
    }
    
    @Override
    protected String paramString() {
        String str = super.paramString() + "," + this.modalityType;
        if (this.title != null) {
            str = str + ",title=" + this.title;
            return str;
        }
        return str;
    }
    
    private static native void initIDs();
    
    void modalShow() {
        IdentityArrayList c = new IdentityArrayList();
        for (Dialog owner_NoClientCode : Dialog.modalDialogs) {
            if (owner_NoClientCode.shouldBlock(this)) {
                for (; owner_NoClientCode != null && owner_NoClientCode != this; owner_NoClientCode = owner_NoClientCode.getOwner_NoClientCode()) {}
                if (owner_NoClientCode == this || !this.shouldBlock(owner_NoClientCode) || this.modalityType.compareTo(owner_NoClientCode.getModalityType()) < 0) {
                    c.add(owner_NoClientCode);
                }
                else {
                    continue;
                }
            }
            else {
                continue;
            }
        }
        for (int i = 0; i < c.size(); ++i) {
            Dialog dialog = (Dialog)c.get(i);
            if (dialog.isModalBlocked()) {
                Dialog modalBlocker = dialog.getModalBlocker();
                if (!c.contains(modalBlocker)) {
                    c.add(i + 1, modalBlocker);
                }
            }
        }
        if (c.size() > 0) {
            ((Dialog)c.get(0)).blockWindow(this);
        }
        IdentityArrayList o = new IdentityArrayList(c);
        for (int j = 0; j < o.size(); ++j) {
            Window[] ownedWindows_NoClientCode = ((Window)o.get(j)).getOwnedWindows_NoClientCode();
            for (int length = ownedWindows_NoClientCode.length, k = 0; k < length; ++k) {
                o.add(ownedWindows_NoClientCode[k]);
            }
        }
        IdentityLinkedList o2 = new IdentityLinkedList();
        for (Window o3 : Window.getAllUnblockedWindows()) {
            if (this.shouldBlock(o3) && !o.contains(o3)) {
                if (o3 instanceof Dialog && ((Dialog)o3).isModal_NoClientCode()) {
                    Dialog o4 = (Dialog)o3;
                    if (o4.shouldBlock(this) && Dialog.modalDialogs.indexOf(o4) > Dialog.modalDialogs.indexOf(this)) {
                        continue;
                    }
                }
                o2.add(o3);
            }
            else {
                continue;
            }
        }
        this.blockWindows(o2);
        if (!this.isModalBlocked()) {
            this.updateChildrenBlocking();
        }
    }
    
    void modalHide() {
        IdentityArrayList<Window> list = new IdentityArrayList();
        int size = this.blockedWindows.size();
        for (int i = 0; i < size; ++i) {
            Window e = (Window)this.blockedWindows.get(0);
            list.add(e);
            this.unblockWindow(e);
        }
        for (int j = 0; j < size; ++j) {
            Window window = (Window)list.get(j);
            if (window instanceof Dialog && ((Dialog)window).isModal_NoClientCode()) {
                ((Dialog)window).modalShow();
            }
            else {
                checkShouldBeBlocked(window);
            }
        }
    }
    
    boolean shouldBlock(Window parent_NoClientCode) {
        if (!this.isVisible_NoClientCode() || (!parent_NoClientCode.isVisible_NoClientCode() && !parent_NoClientCode.isInShow) || this.isInHide || parent_NoClientCode == this || !this.isModal_NoClientCode()) {
            return false;
        }
        if (parent_NoClientCode instanceof Dialog && ((Dialog)parent_NoClientCode).isInHide) {
            return false;
        }
        while (this != null) {
            for (; parent_NoClientCode != null && parent_NoClientCode != this; parent_NoClientCode = parent_NoClientCode.getParent_NoClientCode()) {}
            if (parent_NoClientCode == this) {
                return false;
            }
            this = this.getModalBlocker();
        }
        switch (this.modalityType) {
            case MODELESS: {
                return false;
            }
            case DOCUMENT_MODAL: {
                if (parent_NoClientCode.isModalExcluded(Dialog.ModalExclusionType.APPLICATION_EXCLUDE)) {
                    while (this != null && this != parent_NoClientCode) {
                        this = this.getParent_NoClientCode();
                    }
                    return this == parent_NoClientCode;
                }
                return this.getDocumentRoot() == parent_NoClientCode.getDocumentRoot();
            }
            case APPLICATION_MODAL: {
                return !parent_NoClientCode.isModalExcluded(Dialog.ModalExclusionType.APPLICATION_EXCLUDE) && this.appContext == parent_NoClientCode.appContext;
            }
            case TOOLKIT_MODAL: {
                return !parent_NoClientCode.isModalExcluded(Dialog.ModalExclusionType.TOOLKIT_EXCLUDE);
            }
            default: {
                return false;
            }
        }
    }
    
    void blockWindow(Window e) {
        if (!e.isModalBlocked()) {
            e.setModalBlocked(this, true, true);
            this.blockedWindows.add(e);
        }
    }
    
    void blockWindows(List<Window> c) {
        DialogPeer dialogPeer = (DialogPeer)this.peer;
        if (dialogPeer == null) {
            return;
        }
        Iterator<Object> iterator = c.iterator();
        while (iterator.hasNext()) {
            Window window = (Window)iterator.next();
            if (!window.isModalBlocked()) {
                window.setModalBlocked(this, true, false);
            }
            else {
                iterator.remove();
            }
        }
        dialogPeer.blockWindows(c);
        this.blockedWindows.addAll(c);
        return;
    }
    
    void unblockWindow(Window window) {
        if (window.isModalBlocked() && this.blockedWindows.contains(window)) {
            this.blockedWindows.remove(window);
            window.setModalBlocked(this, false, true);
        }
    }
    
    static void checkShouldBeBlocked(Window window) {
        synchronized (window.getTreeLock()) {
            int i = 0;
            while (i < Dialog.modalDialogs.size()) {
                Dialog dialog = (Dialog)Dialog.modalDialogs.get(i);
                if (dialog.shouldBlock(window)) {
                    dialog.blockWindow(window);
                    break;
                }
                ++i;
            }
        }
    }
    
    private void checkModalityPermission(Dialog.ModalityType modalityType) {
        if (modalityType == Dialog.ModalityType.TOOLKIT_MODAL) {
            SecurityManager securityManager = System.getSecurityManager();
            if (securityManager != null) {
                securityManager.checkPermission(SecurityConstants.AWT.TOOLKIT_MODALITY_PERMISSION);
            }
        }
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws ClassNotFoundException, IOException, HeadlessException {
        GraphicsEnvironment.checkHeadless();
        ObjectInputStream.GetField fields = objectInputStream.readFields();
        Dialog.ModalityType default_MODALITY_TYPE = (Dialog.ModalityType)fields.get("modalityType", null);
        try {
            this.checkModalityPermission(default_MODALITY_TYPE);
        }
        catch (AccessControlException ex) {
            default_MODALITY_TYPE = Dialog.DEFAULT_MODALITY_TYPE;
        }
        if (default_MODALITY_TYPE == null) {
            this.setModal(this.modal = fields.get("modal", false));
        }
        else {
            this.modalityType = default_MODALITY_TYPE;
        }
        this.resizable = fields.get("resizable", true);
        this.undecorated = fields.get("undecorated", false);
        this.title = (String)fields.get("title", "");
        this.blockedWindows = new IdentityArrayList();
        SunToolkit.checkAndSetPolicy(this);
        this.initialized = true;
    }
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if (this.accessibleContext == null) {
            this.accessibleContext = new Dialog.AccessibleAWTDialog();
        }
        return this.accessibleContext;
    }
    
    static {
        Toolkit.loadLibraries();
        if (!GraphicsEnvironment.isHeadless()) {
            initIDs();
        }
        DEFAULT_MODALITY_TYPE = Dialog.ModalityType.APPLICATION_MODAL;
        Dialog.modalDialogs = new IdentityArrayList();
        Dialog.nameCounter = 0;
    }
    
    public enum ModalityType
    {
        MODELESS, 
        DOCUMENT_MODAL, 
        APPLICATION_MODAL, 
        TOOLKIT_MODAL;
    }
    
    public enum ModalExclusionType
    {
        NO_EXCLUDE, 
        APPLICATION_EXCLUDE, 
        TOOLKIT_EXCLUDE;
    }
    
    protected class AccessibleAWTDialog extends Window.AccessibleAWTWindow
    {
        private static final long serialVersionUID = 4837230331833941201L;
        
        @Override
        public AccessibleRole getAccessibleRole() {
            return AccessibleRole.DIALOG;
        }
        
        @Override
        public AccessibleStateSet getAccessibleStateSet() {
            AccessibleStateSet accessibleStateSet = super.getAccessibleStateSet();
            if (Dialog.this.getFocusOwner() != null) {
                accessibleStateSet.add(AccessibleState.ACTIVE);
            }
            if (Dialog.this.isModal()) {
                accessibleStateSet.add(AccessibleState.MODAL);
            }
            if (Dialog.this.isResizable()) {
                accessibleStateSet.add(AccessibleState.RESIZABLE);
                return accessibleStateSet;
            }
            return accessibleStateSet;
        }
    }
}
